︠71ad861d-baf4-48e8-abbc-53e2d6aeb118︠
%sage
# Use a better version of the octave interface
from octave import octave
%default_mode octave
︡823f3785-5b34-44d7-acf6-eed9eaaa1ec5︡︡{"done":true}
︠f486f05b-313b-4966-b18c-41f768b07ce5︠
#MIW2DVoronoiV1
#Fully working dmeonstration of basci tesselation functions and method for generating
#a list of all nearest neighbors of every input node, and a list of all neighbors and their
#mearest neighbors.

#MIW2DVoronoiV1 is next step in evolution of this code.

#graphics_toolkit gnuplot
#graphics_toolkit fltk

#Find a good increment to use to explore points near tails
endval=0.999;
incval=2*endval/20;
xuniform=[-endval:incval:endval]'; %For the simple test
yuniform=[-endval:incval:endval]'; %For the simple test

#For a Gaussian 2D density with zero covariance the density is factorable permitting...
#Use inverse error function to generate a Gaussian marginal distribution of trajectories

xvalues=erfinv(xuniform);
yvalues=erfinv(yuniform);
nxv=size(xvalues,1);
nyv=size(yvalues,1);
N=nxv*nyv; %Number of worlds

#We need to build the 2D coordinates obtained from Cartesian product of x and y coordinates
[Xc, Yc] = meshgrid (xvalues, yvalues);

#X is the state vector, a single column with every x and y coordinate as an entry followed
#by every time derivative of these coordinates 
#For the 2D case our stacking will be of all x coordinates first, then all y, then derivatives
Xcol=reshape(Xc,N,1);
Ycol=reshape(Yc,N,1);
X = [Xcol; Ycol; zeros(2*N,1)]; %adding zero initial velocities
#X = [reshape(Xc,N,1); reshape(Xc,N,1); zeros(2*N,1)]; %adding non-zero initial velocities

#Let's plot the input initial locations of the N worlds and the tesselation of the space
h1=figure('visible', 'off');
tri = delaunay (Xcol, Ycol);
[vx, vy] = voronoi (Xcol, Ycol, tri);
triplot (tri, Xcol, Ycol, "b");
hold on;

#add text annotations for node numbers
offset=0.02; #adjust for readability
text(Xcol+offset,Ycol+offset,num2str([1:N]'));

plot (vx, vy, "r");
axis([-2,2,-2,2]);
hold off;
saveas(h1,"figure1.png");
︡0305eaef-1898-4e74-9c44-e95aceb799d2︡︡{"done":true}
︠af995148-13cf-44c6-b04f-5bb2f6901737︠
%sage salvus.file("figure1.png")
︡387c6df3-8ae7-4c5d-9a55-8fd99059e8fb︡︡{"once":false,"done":false,"file":{"show":true,"uuid":"69334454-5e24-44fc-9e48-04283fee34d3","filename":"figure1.png"}}︡{"done":true}
︠d56d98df-92ea-41ca-b9e4-4d2cfe582b95︠


#The function voronoin will generate two lists:
#C contains the vertices of the Voronoi polygons. 
#F contains, for each original point, the indices of the Voronoi polygon vertices. 
#while polyarea will create the areas of each voronoi polygon for us
[C, F] = voronoin ([Xcol, Ycol]);
af = zeros (size (F));
for i = 1 : length (F)
  af(i) = polyarea (C (F {i, :}, 1), C (F {i, :}, 2));
endfor
#display(af) #display areas

#So the probability density associated with each trajectory point is
Pin=(N*af).^(-1);

%Plot the input trajectory density
#Next 2 lines only needed to avoid plot data error
Pf=Pin;
Pf(isnan(Pin))=0;
︡5f4b4603-d144-409c-9d6d-ca47703721aa︡︡{"done":true}
︠d2d7570e-042c-4615-97b7-07c0a3fa5709︠
%sage
m=matrix(RR,octave("[Xcol(:), Ycol(:), Pf(:)]"))
data=[(x,y,z) for x,y,z in m.rows()]
surface=list_plot3d(data,aspect_ratio=[1,1,8],texture={'color':'green','opacity':0.85,'ambient':0.9})
pts=points(data,size=2,color='red')
#show(pts,aspect_ratio=[1,1,10])
show(surface+pts,spin=10,background='pink',height=int(600),width=int(1000),renderer='webgl',frame={'thickness':4.0, 'color ':'red','labels' : True,'fontface' : 'roman'} )
︡9880f96d-e567-4247-878f-d225459a7efd︡︡{"done":false,"stderr":"/projects/sage/sage-6.10/local/lib/python2.7/site-packages/matplotlib-1.5.0-py2.7-linux-x86_64.egg/matplotlib/cbook.py:136: MatplotlibDeprecationWarning: The matplotlib.delaunay module was deprecated in version 1.4. Use matplotlib.tri.Triangulation instead.\n  warnings.warn(message, mplDeprecation, stacklevel=1)\n"}︡{"done":false,"file":{"uuid":"62895a34-ee3c-4887-a0fc-faa66c7f7eb2","filename":"62895a34-ee3c-4887-a0fc-faa66c7f7eb2.sage3d"}}︡{"html":"<div align='center'></div>","done":false}︡{"done":true}
︠c53792fb-6791-4472-bbfa-e5075a6fa464︠
%sage
m=matrix(RR,octave("[Xcol(:), Ycol(:), Pf(:)]"))
data=[(x,y,z) for x,y,z in m.rows()]
surface=list_plot3d(data,aspect_ratio=[1,1,8],texture="green")
pts=points(data,size=2,color='red')
#show(pts,aspect_ratio=[1,1,10])
show(surface+pts,spin=100,frame=False)
︡54b549e2-7ea0-4e84-87f2-cdd058c4c44f︡︡{"done":false,"file":{"uuid":"d950f949-17a9-47f7-aa5b-d69dbcbf2f54","filename":"d950f949-17a9-47f7-aa5b-d69dbcbf2f54.sage3d"}}︡{"html":"<div align='center'></div>","done":false}︡{"done":true}
︠54506cdc-51f3-4581-b880-ad9caa7ca8fb︠
%sage
show(surface+pts,viewer='tachyon',mesh=True,frame=False,axes=False)
︡d445a9bf-9e6c-4730-96f9-1b89b6d71b9f︡︡{"once":false,"done":false,"file":{"show":true,"uuid":"44218d3c-9097-48ea-8959-37a9f95e265d","filename":"/projects/b04b5777-e269-4c8f-a4b8-b21dbe1c93c6/.sage/temp/compute4-us/18905/tmp_jsXREV.png"}}︡{"html":"<div align='center'></div>","done":false}︡{"done":true}
︠1891e1d8-ef7a-4f21-ab22-140f2bd499cf︠
%sage
show(graphs.DodecahedralGraph(), d3=True, force_spring_layout=True)
︡32adb21d-4375-46f6-9709-9f701d924517︡︡{"d3":{"viewer":"graph","data":{"link_distance":50,"link_strength":1,"links":[{"strength":0,"target":1,"color":"#aaa","curve":0,"source":0,"name":""},{"strength":0,"target":10,"color":"#aaa","curve":0,"source":0,"name":""},{"strength":0,"target":19,"color":"#aaa","curve":0,"source":0,"name":""},{"strength":0,"target":2,"color":"#aaa","curve":0,"source":1,"name":""},{"strength":0,"target":8,"color":"#aaa","curve":0,"source":1,"name":""},{"strength":0,"target":3,"color":"#aaa","curve":0,"source":2,"name":""},{"strength":0,"target":6,"color":"#aaa","curve":0,"source":2,"name":""},{"strength":0,"target":4,"color":"#aaa","curve":0,"source":3,"name":""},{"strength":0,"target":19,"color":"#aaa","curve":0,"source":3,"name":""},{"strength":0,"target":5,"color":"#aaa","curve":0,"source":4,"name":""},{"strength":0,"target":17,"color":"#aaa","curve":0,"source":4,"name":""},{"strength":0,"target":6,"color":"#aaa","curve":0,"source":5,"name":""},{"strength":0,"target":15,"color":"#aaa","curve":0,"source":5,"name":""},{"strength":0,"target":7,"color":"#aaa","curve":0,"source":6,"name":""},{"strength":0,"target":8,"color":"#aaa","curve":0,"source":7,"name":""},{"strength":0,"target":14,"color":"#aaa","curve":0,"source":7,"name":""},{"strength":0,"target":9,"color":"#aaa","curve":0,"source":8,"name":""},{"strength":0,"target":10,"color":"#aaa","curve":0,"source":9,"name":""},{"strength":0,"target":13,"color":"#aaa","curve":0,"source":9,"name":""},{"strength":0,"target":11,"color":"#aaa","curve":0,"source":10,"name":""},{"strength":0,"target":12,"color":"#aaa","curve":0,"source":11,"name":""},{"strength":0,"target":18,"color":"#aaa","curve":0,"source":11,"name":""},{"strength":0,"target":13,"color":"#aaa","curve":0,"source":12,"name":""},{"strength":0,"target":16,"color":"#aaa","curve":0,"source":12,"name":""},{"strength":0,"target":14,"color":"#aaa","curve":0,"source":13,"name":""},{"strength":0,"target":15,"color":"#aaa","curve":0,"source":14,"name":""},{"strength":0,"target":16,"color":"#aaa","curve":0,"source":15,"name":""},{"strength":0,"target":17,"color":"#aaa","curve":0,"source":16,"name":""},{"strength":0,"target":18,"color":"#aaa","curve":0,"source":17,"name":""},{"strength":0,"target":19,"color":"#aaa","curve":0,"source":18,"name":""}],"pos":[],"height":null,"edge_thickness":2,"loops":[],"edge_labels":false,"vertex_size":7,"vertex_labels":true,"directed":false,"gravity":0.04,"width":null,"charge":-120,"nodes":[{"group":"0","name":"0"},{"group":"0","name":"1"},{"group":"0","name":"2"},{"group":"0","name":"3"},{"group":"0","name":"4"},{"group":"0","name":"5"},{"group":"0","name":"6"},{"group":"0","name":"7"},{"group":"0","name":"8"},{"group":"0","name":"9"},{"group":"0","name":"10"},{"group":"0","name":"11"},{"group":"0","name":"12"},{"group":"0","name":"13"},{"group":"0","name":"14"},{"group":"0","name":"15"},{"group":"0","name":"16"},{"group":"0","name":"17"},{"group":"0","name":"18"},{"group":"0","name":"19"}]}}}︡{"html":"<div align='center'></div>","done":false}︡{"done":true}
︠0d6df0ac-9149-4791-ad0d-a07647f6ceb6︠
#The nearest neighbors of a point are the points connected to it in the Delaunay
#tesselation. So, we need need only find its index in the list of indices 
#to find those nearest neighbors

#Useful helper functions: the two functions missing from matlab/octave that no one
#will admit to because the omission is so agregious but the surrogate syntax is so ugly
paren = @(x, varargin) x(varargin{:});
curly = @(x, varargin) x{varargin{:}};

#Find the nearest neighbors of every node
%f=@(x)find(tri==x);
%atemp=arrayfun(f,[1:size(y)],"UniformOutput", false);
%rowlist=arrayfun(@(x)ind2sub(size(tri),atemp{x}),[1:size(y)],"UniformOutput", false);
%nlist=arrayfun(@(x) setdiff(unique(tri(rowlist{x},:)),x),[1:size(y)],"UniformOutput", false)

#Find the nearest neighbors of all nodes

#ftri finds the rows with nearest neighbors (nns) of node given in argument
ftri=@(x)find(tri==x);
#ftmany applies ftri to each node in narray to find linear index of entries in tri with its nns
ftmany=@(narray)arrayfun(ftri,narray,"UniformOutput", false);
#Generate an array listing all the rows discovered delinearizing entries found by ftmany
rowlist=arrayfun(@(x)ind2sub(size(tri),curly(ftmany([1:N]),x)),[1:N],"UniformOutput", false);
#Find the unique nodes listed by all rows found for each node in node ordered array found above
nlist=arrayfun(@(x) setdiff(unique(tri(rowlist{x},:)),x),[1:N],"UniformOutput", false);

#Find the nns and the nearest neighbors of the nearest neighbors of all nodes

clear nnslist
nnlist=cell(N);
for nlnum=1:N
  ntemp=nlist{nlnum}';
  #Generate an array listing all the rows discovered delinearizing entries found by ftmany
  nnsrowlist=arrayfun(@(x)ind2sub(size(tri),curly(ftmany(ntemp),x)),[1:size(ntemp)],"UniformOutput", false);
  nnsrowmerge=vertcat(nnsrowlist{:});
  nnslist{nlnum}=unique(tri(nnsrowmerge,:));
endfor

#Now fit a 2nd order 2d polynomial to each region surrounding a point
clear nnspolyval;
#nnspolyval will store the interpolated values of [P,Px,Py,Pxx,Pxy,Pyy]
nnspolyval=zeros(N,6);

#Define functions that enact differentiation of the polynomial on the coeffs
Dx=@(x) [x(2) 2*x(3) 0 x(5) 0 0]';
Dy=@(x) [x(4) x(5) 0 2*x(6) 0 0]';
Dxx=@(x) [2*x(3) 0 0 0 0 0]';
Dyy=@(x) [2*x(6) 0 0 0 0 0]';
Dxy=@(x) [x(5) 0 0 0 0 0]';

for nlnum=1:N
  nnspoly = polyfit2d(Xcol(nnslist{nlnum}), Ycol(nnslist{nlnum}), Pf(nnslist{nlnum}), 2);
  xyprods=[1,Xcol(nlnum),Xcol(nlnum)^2,Ycol(nlnum),Xcol(nlnum)*Ycol(nlnum),Ycol(nlnum)^2];
  #nnspolyval(nlnum,1)=polyval2d(Xcol(nlnum),Ycol(nlnum),nnspoly);
  nnspolyval(nlnum,1)=abs(xyprods*nnspoly); #abs to eliminate small negative outcomes
  nnspolyval(nlnum,2)=xyprods*Dx(nnspoly); 
  nnspolyval(nlnum,3)=xyprods*Dy(nnspoly); 
  nnspolyval(nlnum,4)=xyprods*Dxx(nnspoly); 
  nnspolyval(nlnum,5)=xyprods*Dxy(nnspoly); 
  nnspolyval(nlnum,6)=xyprods*Dyy(nnspoly); 
endfor;
︡fe29a613-9ebb-49b7-aaab-8343ca55e7ce︡︡{"stdout":"error: condest: matrix must be square\nerror: called from\n    condest at line 130 column 7\n    polyfit2d at line 65 column 5\n    /projects/b04b5777-e269-4c8f-a4b8-b21dbe1c93c6/.sage/temp/compute4-us/18905/interface/tmp18975 at line 52 column 11","done":false}︡{"done":true}
︠5b3b148e-effe-46f0-8ff7-d13edffb8823︠
%sage
m=matrix(RR,octave("[Xcol(:), Ycol(:), nnspolyval(:,1)]"))
list_plot3d([(x,y,z) for x,y,z in m.rows()],texture="blue",frame_aspect_ratio=[1,1,10])
︡bd1cbab9-139b-4335-b6bb-e9d5e87647c2︡︡{"done":false,"file":{"uuid":"2c75516c-fbde-48d3-9389-39142683f1b6","filename":"2c75516c-fbde-48d3-9389-39142683f1b6.sage3d"}}︡{"html":"<div align='center'></div>","done":false}︡{"done":true}
︠c4499ed5-e3ae-4662-afe2-fafa980a0474︠
h3=figure('visible', 'off');
clf;
#Use below for gnuplot and following line for full gradient color with fltk
g=patch ("Faces", tri, "Vertices", [Xcol(:), Ycol(:), nnspolyval(:,1)]);
#patch ("Faces", tri, "Vertices", [Xcol(:), Ycol(:), Pin(:)],'FaceVertexCData',Pin(:),'FaceColor','interp','EdgeColor','none');
view (359, 11);
set(g,'edgecolor','b');
︡5c4043b7-d907-451f-84b2-b39a307d92fb︡︡{"done":true}
︠e56b8452-39f8-4614-99de-f88ccd54048a︠
%sage
m=matrix(RR,octave("[Xcol(:), Ycol(:), nnspolyval(:,2)]"))
data=[(x,y,z) for x,y,z in m.rows()]
surface=list_plot3d(data,aspect_ratio=[1,1,8],texture="green")
pts=points(data,size=2,color='red')
surface+pts
︡85dff5a8-a97a-4811-be8e-299e0fcb2cec︡︡{"done":false,"file":{"uuid":"eb9e8a14-f99b-4033-8e0e-9cce913594aa","filename":"eb9e8a14-f99b-4033-8e0e-9cce913594aa.sage3d"}}︡{"html":"<div align='center'></div>","done":false}︡{"done":true}
︠e7a51805-f111-44f7-9b4e-7fd274b751f2︠

h4=figure('visible', 'off');
clf;
#Use below for gnuplot and following line for full gradient color with fltk
patch ("Faces", tri, "Vertices", [Xcol(:), Ycol(:), nnspolyval(:,2)]);
#patch ("Faces", tri, "Vertices", [Xcol(:), Ycol(:), Pin(:)],'FaceVertexCData',Pin(:),'FaceColor','interp','EdgeColor','none');
view (359, 11);
︡6139b673-18f0-41ef-92ee-e4431c04bda9︡︡{"done":true}
︠de82dcf9-0f8c-4f3b-ae81-b27b58cee1fe︠
%sage
m=matrix(RR,octave("[Xcol(:), Ycol(:), nnspolyval(:,3)]"))
data=[(x,y,z) for x,y,z in m.rows()]
surface=list_plot3d(data,aspect_ratio=[1,1,8],texture="lightgreen")
pts=points(data,size=2,color='green')
surface+pts
︡ad8e6be4-2bbd-41ce-9fe6-6edf6182b361︡︡{"done":false,"file":{"uuid":"5734509b-e52e-46ae-b8dd-2a38fb759c7e","filename":"5734509b-e52e-46ae-b8dd-2a38fb759c7e.sage3d"}}︡{"html":"<div align='center'></div>","done":false}︡{"done":true}
︠44ab34f2-bbeb-4866-b93c-ac1fd4e3e8fb︠

h5=figure('visible', 'off');
clf;
#Use below for gnuplot and following line for full gradient color with fltk
patch ("Faces", tri, "Vertices", [Xcol(:), Ycol(:), nnspolyval(:,3)]);
#patch ("Faces", tri, "Vertices", [Xcol(:), Ycol(:), Pin(:)],'FaceVertexCData',Pin(:),'FaceColor','interp','EdgeColor','none');
view (246, 20);
︡1797bce1-fb0e-40b5-8f22-e2d1963ca62e︡︡{"done":true}
︠b1883585-949d-49e0-bd14-790e7ecbbb87︠
︠849ab3f8-aab2-4992-9e32-ae26ee6a02b6︠
saveas(h3,'figureVDensSmooth.png');
saveas(h4,'figureVDensDx.png');
saveas(h5,'figureVDensDy.png');
︡97b43b11-7bbd-43af-9f3c-1390a048c76b︡︡{"stdout":"Can't find PostScript prologue file prologue.ps\n\tloadpath is empty\n\tgnuplotrc is read from /projects/24a70e9b-9eee-4d22-8665-b589c410fa4b/share/gnuplot/5.0\n\tno XAPPLRESDIR found in the environment,\n\t    falling back to \"/etc/X11/app-defaults/\"\nPlease copy prologue.ps to one of the above directories\nor set the environmental variable GNUPLOT_PS_DIR\nor set the loadpath appropriately\n         line 0: Plot failed!\n\nCan't find PostScript prologue file prologue.ps\n\tloadpath is empty\n\tgnuplotrc is read from /projects/24a70e9b-9eee-4d22-8665-b589c410fa4b/share/gnuplot/5.0\n\tno XAPPLRESDIR found in the environment,\n\t    falling back to \"/etc/X11/app-defaults/\"\nPlease copy prologue.ps to one of the above directories\nor set the environmental variable GNUPLOT_PS_DIR\nor set the loadpath appropriately\n         line 5599: Plot failed!\n\nCan't find PostScript prologue file prologue.ps\n\tloadpath is empty\n\tgnuplotrc is read from /projects/24a70e9b-9eee-4d22-8665-b589c410fa4b/share/gnuplot/5.0\n\tno XAPPLRESDIR found in the environment,\n\t    falling back to \"/etc/X11/app-defaults/\"\nPlease copy prologue.ps to one of the above directories\nor set the environmental variable GNUPLOT_PS_DIR\nor set the loadpath appropriately\n         line 0: Plot failed!\n\nCan't find PostScript prologue file prologue.ps\n\tloadpath is empty\n\tgnuplotrc is read from /projects/24a70e9b-9eee-4d22-8665-b589c410fa4b/share/gnuplot/5.0\n\tno XAPPLRESDIR found in the environment,\n\t    falling back to \"/etc/X11/app-defaults/\"\nPlease copy prologue.ps to one of the above directories\nor set the environmental variable GNUPLOT_PS_DIR\nor set the loadpath appropriately\n         line 5599: Plot failed!\n\nCan't find PostScript prologue file prologue.ps\n\tloadpath is empty\n\tgnuplotrc is read from /projects/24a70e9b-9eee-4d22-8665-b589c410fa4b/share/gnuplot/5.0\n\tno XAPPLRESDIR found in the environment,\n\t    falling back to \"/etc/X11/app-defaults/\"\nPlease copy prologue.ps to one of the above directories\nor set the environmental variable GNUPLOT_PS_DIR\nor set the loadpath appropriately\n         line 0: Plot failed!\n\nCan't find PostScript prologue file prologue.ps\n\tloadpath is empty\n\tgnuplotrc is read from /projects/24a70e9b-9eee-4d22-8665-b589c410fa4b/share/gnuplot/5.0\n\tno XAPPLRESDIR found in the environment,\n\t    falling back to \"/etc/X11/app-defaults/\"\nPlease copy prologue.ps to one of the above directories\nor set the environmental variable GNUPLOT_PS_DIR\nor set the loadpath appropriately\n         line 5599: Plot failed!\n","done":false}︡{"done":true}
︠cd2dd256-3666-497f-83d2-e15560cf4acb︠
%sage salvus.file("figureVDensSmooth.png")
︡2479d86f-7ed7-4690-a9c5-236fe7f3792b︡{"once":false,"file":{"show":true,"uuid":"5ad5d871-af4f-4337-881d-dff3059f7d33","filename":"figureVDensSmooth.png"}}︡
︠dbebc849-f612-4e15-85a3-d918c505b7a4︠
%sage salvus.file("figureVDensDx.png")
︡a4b80807-91fa-4a2e-8181-4cf4057bdd42︡{"once":false,"file":{"show":true,"uuid":"8f58113e-fef7-411f-ba2a-7e4ba5d24ab5","filename":"figureVDensDx.png"}}︡
︠5fee3bfe-11b6-4b2e-b391-2b2de385b7ba︠
%sage salvus.file("figureVDensDy.png")
︡73a72408-e8df-426c-bd3d-5acd734142a0︡{"once":false,"file":{"show":true,"uuid":"d7bdd6cb-7ee5-4b57-ab69-e7238d05f289","filename":"figureVDensDy.png"}}︡
︠5e701ea3-249e-41bc-8a32-8487b11c0dd0︠
︡1bf40ddf-3eab-47d4-9e02-24446045e0b0︡
︠0817846a-9e2b-4972-90cc-dabfcded9e46︠









